/** *****************************************************
 * Copyright (C) HRider in motion S.L.
 *
 * This file is part of HRider.
 *
 * HRider can not be copied and/or distributed without the express
 * permission of HRider in motion S.L.
 ****************************************************** */
package net.hrider.api.usecases;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.LogManager;
import java.util.logging.Logger;
import net.hrider.api.client.ApiClient;
import net.hrider.api.client.ApiException;
import net.hrider.api.client.HRiderService;
import net.hrider.api.model.Company;
import net.hrider.api.model.CompanyState;
import net.hrider.api.model.Department;
import net.hrider.api.model.Employee;
import net.hrider.api.model.Language;
import net.hrider.api.model.Link;
import net.hrider.api.model.Position;
import net.hrider.api.model.Report;
import net.hrider.api.model.Root;

/**
 *
 * @author drh (dani@hrider.net)
 */
public abstract class AbstractUseCase {

    public static final String VERSION_PATH = "v1";

    protected HRiderService service;
    protected Root root;
    private Company company;

    private static Logger LOG;

    static {
        try (InputStream in = Thread.currentThread().getContextClassLoader().getResourceAsStream("logging.properties")) {
            LogManager.getLogManager().readConfiguration(in);
            LOG = Logger.getLogger(AbstractUseCase.class.getName());
            LOG.fine("Logs initialized ...");
        } catch (IOException ex) {
            LOG.log(Level.SEVERE, "error reading initializing logger", ex);
        }
    }

    public AbstractUseCase() {
    }

    public void run() {
        ApiClient client = new ApiClient();
        service = new HRiderService(client);
        try {
            root = service.get(Root.class, new Link(VERSION_PATH));
            doRun();
        } catch (ApiException ex) {
            LOG.log(Level.SEVERE, "API error", ex);
        } finally {
            if (company != null) {
                try {
                    service.delete(company.getLinks().getSelf());
                } catch (Exception ex) {
                    LOG.log(Level.SEVERE, "error deleting company " + company, ex);
                }
            }
        }
    }
    
    protected abstract void doRun() throws ApiException;

    // protected methods *******************************************************
    
    protected Company getDummyCompany() {
        if (company == null) {
            try {
                company = service.create(root.getLinks().getCompanies(), buildCompany());
            } catch (ApiException ex) {
                LOG.log(Level.SEVERE, "error creating dummy company", ex);
            }
        }
        return company;
    }

    protected Company buildCompany() {
        Company c = new Company();
        c.setAddress("Acme Street, 17, New York");
        c.setLanguage(Language.EN);
        c.setLimit(50);
        c.setName("Acme");
        c.setState(CompanyState.ACTIVE);
        c.setTaxIdNumber("1111111R");
        c.setTimeZone("America/New_York");
        return c;
    }

    protected Department buildDepartment(String code, String name) {
        Department dpt = new Department();
        dpt.setCode(code);
        dpt.setName(name);
        return dpt;
    }

    protected Position buildPosition(String code, String name, int required) {
        Position p = new Position();
        p.setCode(code);
        p.setName(name);
        p.setRequired(required);
        return p;
    }

    protected Employee buildEmployee(String code, String firstName, String email) {
        Employee emp = new Employee();
        emp.setCode(code);
        emp.setFirstName(firstName);
        emp.setLastName("lastName");
        emp.setEmail(email);
        return emp;
    }

    protected void printEmployees(List<Employee> list) {
        // print tabular data
        StringBuilder out = new StringBuilder();
        String format = "%-20s%-10s%-20s%-20s";
        out.append(String.format(format, "ID", "Code", "Email", "Name")).append("\n");
        out.append(String.format(format, "-------------------", "---------", "------------------", "------------------")).append("\n");
        list.forEach((e) -> {
            out.append(String.format(format, e.getId(), e.getCode(), e.getEmail(), e.getFirstName())).append("\n");
        });
        System.out.println(out);
    }

    protected void printReports(List<Report> list) {
        StringBuilder out = new StringBuilder();
        String format = "%-30s%-42s%-20s";
        out.append(String.format(format, "ID", "Description", "Mimetypes")).append("\n");
        out.append(String.format(format, "---------------------------", "---------------------------------------", "--------------------")).append("\n");
        list.forEach((r) -> {
            StringBuilder mtypes = new StringBuilder(r.getMimetypes()[0]);            
            for (int i = 1; i < r.getMimetypes().length; i++) {
                mtypes.append(",").append(r.getMimetypes()[i]);
            }
            String desc = r.getDescription();
            if (desc.length() > 40) {
                desc = desc.substring(0, 37) + "...";
            }
            out.append(String.format(format, r.getId(), desc, mtypes)).append("\n");
        });
        System.out.println(out);
    }
    
    protected int saveToFile(InputStream in, FileOutputStream fout) throws IOException {
        byte[] buffer = new byte[16 * 1024];
        int l;
        int total = 0;
        while ((l = in.read(buffer)) != -1) {
            fout.write(buffer, 0, l);
            total += l;
        }
        return total;
    }

    // private methods *********************************************************
    

}
